import 'package:comunic/enums/post_target.dart';
import 'package:comunic/helpers/posts_helper.dart';
import 'package:comunic/models/advanced_group_info.dart';
import 'package:comunic/ui/routes/main_route/main_route.dart';
import 'package:comunic/ui/screens/group_members_screen.dart';
import 'package:comunic/ui/screens/group_settings_screen.dart';
import 'package:comunic/ui/widgets/group_following_widget.dart';
import 'package:comunic/ui/widgets/group_icon_widget.dart';
import 'package:comunic/ui/widgets/group_membership_widget.dart';
import 'package:comunic/ui/widgets/like_widget.dart';
import 'package:comunic/ui/widgets/post_create_form_widget.dart';
import 'package:comunic/ui/widgets/posts_list_widget.dart';
import 'package:comunic/ui/widgets/safe_state.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:flutter/material.dart';

/// Authorized group page screen
///
/// This screen is shown when the user is allowed to access to a group's page
///
/// @author Pierre Hubert

enum _MenuItems { OPEN_MEMBERS, OPEN_SETTINGS }

Color get _headerTextColor => Colors.white;

Color get _headerColor => Colors.blueAccent.shade700;

class AuthorizedGroupPageScreen extends StatefulWidget {
  final AdvancedGroupInfo advancedGroupInfo;
  final Function() needRefresh;

  const AuthorizedGroupPageScreen({
    Key key,
    @required this.advancedGroupInfo,
    @required this.needRefresh,
  })  : assert(advancedGroupInfo != null),
        assert(needRefresh != null),
        super(key: key);

  @override
  _AuthorizedGroupPageScreenState createState() =>
      _AuthorizedGroupPageScreenState();
}

class _AuthorizedGroupPageScreenState
    extends SafeState<AuthorizedGroupPageScreen>
    with SingleTickerProviderStateMixin {
  AdvancedGroupInfo get _group => widget.advancedGroupInfo;

  TabController _tabController;

  final _postsKey = GlobalKey<PostsListWidgetState>();

  List<_GroupPageTab> get _tabs => [
        _GroupPageTab(widget: _buildGroupPagePostsList(), label: tr("Posts")),
      ];

  @override
  void initState() {
    _tabController = TabController(
      length: _tabs.length,
      initialIndex: 0,
      vsync: this,
    );

    super.initState();
  }

  @override
  void dispose() {
    _tabController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        _buildGroupPageHeader(),
        Container(
          color: _headerColor,
          child: TabBar(
            tabs: _tabs.map((e) => e.tab).toList(),
            controller: _tabController,
          ),
        ),
        Expanded(
          child: TabBarView(
            controller: _tabController,
            children: _tabs.map((e) => e.widget).toList(),
          ),
        ),
      ],
    );
  }

  /// Build group page header
  Widget _buildGroupPageHeader() {
    return DefaultTextStyle(
      style: TextStyle(color: _headerTextColor),
      child: Container(
        color: _headerColor,
        child: Padding(
          padding: const EdgeInsets.all(8.0),
          child: Row(
            children: <Widget>[
              GroupIcon(
                group: _group,
              ),
              Expanded(
                child: Text(
                  " ${_group.displayName}",
                  style: TextStyle(fontSize: 20),
                ),
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  GroupMembershipWidget(
                    group: _group,
                    onUpdated: () => widget.needRefresh(),
                  ),
                  Container(
                    height: 4,
                  ),
                  GroupFollowingWidget(
                    group: _group,
                    onUpdated: () => widget.needRefresh(),
                    inactiveColor: Colors.blueAccent,
                    activeColor: Colors.white,
                  ),
                  Container(
                    height: 2,
                  ),
                  LikeWidget(
                    inativeColor: Colors.blueAccent,
                    activeColor: Colors.white,
                    likeElement: _group,
                  ),
                ],
              ),
              PopupMenuButton<_MenuItems>(
                itemBuilder: (c) => [
                  PopupMenuItem(
                    child: Text(tr("Group members")),
                    value: _MenuItems.OPEN_MEMBERS,
                    enabled: _group.isAtLeastModerator,
                  ),
                  PopupMenuItem(
                    child: Text(tr("Group settings")),
                    value: _MenuItems.OPEN_SETTINGS,
                    enabled: _group.isAdmin,
                  ),
                ],
                onSelected: _handleMenuSelection,
              ),
            ],
          ),
        ),
      ),
    );
  }

  /// Add create post target
  Widget _buildPostCreationArea() {
    if (!_group.canCreatePost) return Container();

    return PostCreateFormWidget(
        postTarget: PostTarget.GROUP_PAGE,
        targetID: _group.id,
        onCreated: () => _postsKey.currentState.loadPostsList(getOlder: false));
  }

  /// Build the list of posts of the group
  Widget _buildGroupPagePostsList() {
    return PostsListWidget(
      key: _postsKey,
      topWidgets: <Widget>[
        _buildPostCreationArea(),
      ],
      getPostsList: () => PostsHelper().getGroupPosts(_group.id),
      showPostsTarget: false,
      userNamesClickable: true,
      getOlder: (from) => PostsHelper().getGroupPosts(_group.id, from: from),
    );
  }

  /// Handles menu selection
  void _handleMenuSelection(_MenuItems item) {
    switch (item) {
      case _MenuItems.OPEN_MEMBERS:
        MainController.of(context).push(GroupMembersScreen(groupID: _group.id));
        break;

      case _MenuItems.OPEN_SETTINGS:
        MainController.of(context)
            .push(GroupSettingsScreen(groupID: _group.id));
        break;
    }
  }
}

class _GroupPageTab {
  final Widget widget;
  final String label;

  const _GroupPageTab({
    @required this.widget,
    @required this.label,
  })  : assert(widget != null),
        assert(label != null);

  Tab get tab => Tab(text: label);
}
